/*** 
 * \file gre/3dpwter.h
 * \brief GRE::PIECEWISETIN definitions
 *
 * \if NODOC
 * $Id: 3dpwter.h_v 1.13 2013/02/15 20:54:04 mju Exp $
 *
 * $Log: 3dpwter.h_v $
 * Revision 1.13  2013/02/15 20:54:04  mju
 * Fix dox hdr.
 *
 * Revision 1.12  2011/10/06 15:20:43  mju
 * Remove dependencies.
 *
 * Revision 1.1  2003/12/22 17:39:09  vdronov
 * Initial revision
 * \endif
**/ 

#ifndef	INC_GRE_3DPWTER_H
#define	INC_GRE_3DPWTER_H

#ifndef	INC_GRE_3DTERRA_H
#include <gre/3dterra.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif


namespace GRE {

//! Sparce triangulation terrain model.
class PIECEWISETIN : public TERRAIN {
	public:

		PIECEWISETIN (												//! Constructor 
			);

		~PIECEWISETIN (											//! Destructor 
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DRECT3D ALIGN16(m_Extents);
		SPATREF::COORDREFSYS m_CoordRefSys;
		TRANS2D_MAPGEN m_Transformation;

		RVC::RASTER m_Raster;
		GRE::LAYER* m_Layer;

		TRIANGULATION m_Triangulation;
		DOUBLE_ARRAY<DPOINT3D> m_Nodes;
		SIMPLE_ARRAY<EDGEINDEX> m_Edges;						
		BITSET m_EdgesBits;	
		SIMPLE_ARRAY<TRIANGLEINDEX> m_Triangles;			
		BITSET m_TrianglesBits;	
		SCENE3D *m_Scene;

		PIECEWISETIN& operator= (								//! Assignment operator (not implemented)
			const PIECEWISETIN& rhs
			);

		TERRAIN* v_CreateInstance (
			) { return new PIECEWISETIN(); };

		ERRVALUE v_Build (
			const GRE::LAYER *layer
			);

		bool v_IsObjectTypeSupported (
			const RVC::OBJTYPE objtype
			) const { return (objtype == RVC::OBJTYPE_Raster); };

		void v_Free (
			);

		ERRVALUE Create (					 
			const GRE::LAYER *layer
			);

		void Delete (					 
			);

		ERRVALUE v_GetDefaultScene ( 
			SCENE3D& scene
			) const;

		ERRVALUE v_SetScene (
			const SCENE3D& scene,
			const SIMPLE_ARRAY<TEXTURE*> &textures	  
			);

		ERRVALUE v_DrawScene (
			CONTROLLER &controller,			 
			DRAWINGCONTEXT3D& drawing,
			TEXTURE* texture,
			TEXTUREFILTER* texturefilter
			);

		ERRVALUE v_SetScene (
			const SCENE3D& scene,
			const SIMPLE_ARRAY<GEOMETRIC3D*> &geometrics,
			const bool wireframe
			);

		float v_GetElevation (
			const float x,
			const float y
			) const;

		ERRVALUE v_ComputeProfile (
			const POLYLINE& polyline,
			POLYLINE& profile,
			SIMPLE_ARRAY<INT32>& index
			);

		float v_GetAngleToNorth (
			const float x,
			const float y
			) const;

		void v_SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		const SPATREF::COORDREFSYS v_GetCoordRefSys (
			) const { return m_CoordRefSys; };

		const DRECT3D& v_GetExtents (
			) { return m_Extents; };

		void v_CreateBoundary (
			REGION2D& boundary
			);

		bool v_HasNull (
			) const { return false; };

		bool v_IsNull (
			const float x,
			const float y
			) const { return false; };

		ERRVALUE CreateRegularTIN (
			);

		void DeleteRegularTIN (
			);
		ERRVALUE SetSceneForRegularTIN (
			const SCENE3D& scene,
			const SIMPLE_ARRAY<TEXTURE*> &textures	  
			);

		ERRVALUE DrawSceneForRegularTIN (
			CONTROLLER &controller,			 
			DRAWINGCONTEXT3D& drawing,
			TEXTURE* texture,
			TEXTUREFILTER* texturefilter
			);

		void TransformRegularTIN (
			const TRANS2D_MAPGEN& transformation
			);

	#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

};		//! End of namespace

#endif

