/**
 * \file gre/base.h <gre/base.h>
 * \brief Base GRE definitions
 *
 * \if NODOC
 * $Id: base.h_v 1.78 2011/12/08 18:50:24 mju Exp $
 *
 * $Log: base.h_v $
 * Revision 1.78  2011/12/08 18:50:24  mju
 * Move stereoview to own hdr in global namespace.
 *
 * Revision 1.77  2011/11/07 15:41:35  mju
 * Add drawflag_NoSetStyle.
 *
 * Revision 1.76  2011/11/03 21:00:35  mju
 * Add drawflags for AllElements, NoCheckColorLock, NoNotifyDrawElement.
 *
 * Revision 1.75  2011/10/06 15:21:53  mju
 * Add stereoview enum, remove obsolete defns, add 3d forward declarations.
 *
 * Revision 1.74  2011/10/06 13:23:56  mju
 * Add texture and scene3d forward declarations.
 *
 * Revision 1.72  2011/04/06 16:58:28  mju
 * Add uicolor_TextHasHighlightedElements.
 *
 * Revision 1.71  2011/02/22 21:21:39  mju
 * Add uicolor_TextExcludedFromGroupExtent.
 *
 * Revision 1.51  2006/09/21 22:00:42  mju
 * Move element parameters and their required headers to elemparms hdr.
 *
 * Revision 1.44  2006/03/31 16:14:46  mju
 * Had red/blue swapped in uicolor values.
 *
 * Revision 1.41  2006/01/10 21:27:30  mju
 * Add uicolor defns.
 *
 * Revision 1.1  2003/06/16 19:39:13  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_BASE_H
#define	INC_GRE_BASE_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward-declare classes.

namespace GRE {
	struct AUTOLABELPARMS;
	class CTRL_TREEVIEW;
	struct DATATIPPARMS;
	class ELEMSAMPLE_DC;
	class GEOMETRIC3D;
	class GROUP;
	class GROUPVIEW;
	struct LABELPARMS;
	class LABELPLACE;
	class LABELPLACEMENT;
	class LAYER;
	class LAYER_CAD;
	class LAYER_FORMULA;
	class LAYER_LEGEND;
	class LAYER_LEGEND2;
	class LAYER_MAPGRID;
	class LAYER_PINMAP;
	class LAYER_RASTER;
	class LAYER_REGION;
	class LAYER_SCALEBAR;
	class LAYER_SCRIPT;
	class LAYER_SHAPE;
	class LAYER_SKETCH;
	class LAYER_SURFACE;
	class LAYER_TEXT;
	class LAYER_TIN;
	class LAYER_VECTOR;
	class LAYERDC;
	class LAYERSECTION;
	class LAYERVIEW;
	class LAYOUT;
	class LEGEND;
	struct LEGENDPARMS;
	struct LINEPARMS;
	class MACROSCRIPT;
	class MACROSCRIPTLIST;
	class MSG;
	class OBJECT;
	struct POINTPARMS;
	struct POLYPARMS;
	class SCENE3D;
	class SCRIPTCONTEXT_DRAW;
	class SCRIPTCONTEXT_STYLE;
	struct SELECTPARMS;
	struct STYLEPARMS;
	class TERRAIN;
	class TEXTURE;
	class TOOL;
	class TOOLSCRIPT;
	class TOOLSCRIPTLIST;
	class VIEW;
	class VIEW_LEGACY;
	class VIEWABLE;
	class VIEWPOINT3D;
	class WIREFRAME;
	}

#endif // GENERATING_DOXYGEN_OUTPUT


#define	GRE_SERIAL_VERSION	3

namespace GRE {

//! Position in list for use in creation and reordering.
enum LISTPOS {
	LISTPOS_First =	0,
	LISTPOS_After =	1,
	LISTPOS_Before =	2,
	LISTPOS_Last =		3
	};

//! Condition for redrawing views.
enum DRAWCONDITION {
	DRAWCONDITION_Always,			//!< Always draw
	DRAWCONDITION_IfNeeded,			//!< Only draw if has changed
	DRAWCONDITION_IfAutoUpdate		//!< Only draw if auto-update preference is on and has changed
	};

//! Drawing flags.
enum DRAWFLAGS {
	DRAWFLAG_None =						0x00000000,
	DRAWFLAG_ViewInView =				0x00000001,	//!< Draw to view-in-view buffer
	DRAWFLAG_OnlyHighlighted =			0x00000002,	//!< Only draw 'highlighted' elements
	DRAWFLAG_AllElements =				0x00000004,	//!< All elements are being drawn
	DRAWFLAG_RequestRemote =			0x00000008,	//!< Requesting remote (web, etc) object
	DRAWFLAG_Erase =						0x00000010,	//!< Erase elements (for GRE::VIEW::DrawLayerElement/s)
	DRAWFLAG_ForceDraw =					0x00000020,	//!< Force all specified elements to draw even if not "selected"
	DRAWFLAG_UpdateSearchTree =		0x00000040,	//!< Update search tree when drawing even if in tree already
	DRAWFLAG_UpdateScreen =				0x00000080,	//!< Refresh screen after drawing
	DRAWFLAG_NoUpdateScreen =			0x00000100,	//!< Disable screen refresh after drawing
	DRAWFLAG_NoHighlighting =			0x00000200,	//!< Draw elements without highlighting
	DRAWFLAG_UpdateBuffers =			0x00000400,	//!< Update render buffers
	DRAWFLAG_SelectChanged =			DRAWFLAG_UpdateBuffers,
	DRAWFLAG_NoSetStyle =				0x00000800,	//!< Don't need to set style when drawing element
	DRAWFLAG_AllNonLocViews =			0x00001000,	//!< Draw all non-locator views
	DRAWFLAG_NoReportDone =				0x00002000,	//!< Don't display message when done drawing
	DRAWFLAG_NoMessage =					0x00004000,	//!< Don't display messages while drawing
	DRAWFLAG_Wireframe =					0x00008000,	//!< Only draw wireframes
	DRAWFLAG_NoResumeTool =				0x00010000,	//!< Don't "resume" tool after drawing view
	DRAWFLAG_NoNotifyDrawElement =	0x00020000,	//!< Don't call NotifyDrawElementBegin/End
	DRAWFLAG_NoBlankScreen =			0x00040000,	//!< Don't "blank" screen when start drawing view
	DRAWFLAG_NoCheckColorLock =		0x00080000,	//!< Don't do colorlock check when drawing element
	DRAWFLAG_NoCallbacks =				0x00400000,	//!< Don't invoke callback lists while drawing
	DRAWFLAG_CheckDrawnSet =			0x00800000,	//!< Check "drawn" set for highlighting
	DRAWFLAG_ReduceSpeckle3D =			0x01000000,	//!< Perform 3D background speckle reduction
	DRAWFLAG_UseDataTipColor =			0x02000000,	//!< Draw in 'datatip element' color.
	DRAWFLAG_UseHighlightColor =		0x04000000,	//!< Draw elements in "highlight" color
	DRAWFLAG_UseActiveColor =			0x08000000,	//!< Draw element in "active" color
	DRAWFLAG_NoCheckVisible =			0x10000000,	//!< Don't check visiblity of layer/group in view
	DRAWFLAG_LabelPlacementPass =		0x20000000,	//!< Internal use: 2nd pass to render just labels
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DRAWFLAGS)
#endif

//! Flags for "find" methods.
enum FINDFLAGS {
	FINDFLAG_ViewCoords =		0x0000,	//!< Point given in "view" coordinates
	FINDFLAG_DisplayCoords =	0x0001,	//!< Point given in "display" coordinates
	FINDFLAG_LayerCoords =		0x0002,	//!< Point given in "layer" (object) coordinates
	FINDFLAG_UseClosest =		0x0004,	//!< Return closest if no other found
	FINDFLAG_NoSingleLayer =	0x0008,	//!< Don't check single-layer groups
	FINDFLAG_No3D =				0x0010,	//!< Don't check 3-D groups
	FINDFLAG_SearchDrawnSet =	0x0020,	//!< Only search the "drawn" element set
	FINDFLAG_MapCoords =			0x0040	//!< Point given in "map" coordinates
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FINDFLAGS)
#endif

//! Targets for rendering individual elements.
enum RENDERTARGET {
	RENDERTARGET_None =					0x0000,
	RENDERTARGET_LayerNormal =			0x0001,		//!< Layer buffer for unhighlighted elements
	RENDERTARGET_LayerHighlight =		0x0002,		//!< Layer buffer for highlighted elements
	RENDERTARGET_LayerLabel =			0x0004,		//!< Layer buffer for labels
	RENDERTARGET_LayerAll =				0x0007,		//!< All layer buffers
	RENDERTARGET_Group =					0x0010,		//!< Group combined buffer
	RENDERTARGET_GroupActiveElems =	0x0020,		//!< Group 'active elements' buffer
	RENDERTARGET_GroupMatte =			0x0040,		//!< Group matte buffer
	RENDERTARGET_GroupAll =				0x0070,		//!< All group buffers
	RENDERTARGET_ViewMemory =			0x1000,		//!< View memory buffer
	RENDERTARGET_ViewScreen =			0x8000		//!< View screen (unbuffered)
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(RENDERTARGET)
#endif

//! Mode for highlighing elements.
enum HIGHLIGHTMODE {
	HIGHLIGHTMODE_NotSet =		0,		//!< Not set, use parent's setting
	HIGHLIGHTMODE_Replace =		1,		//!< Replace selected set with elements provided
	HIGHLIGHTMODE_Add =			2,		//!< Add elements provided to selected set
	HIGHLIGHTMODE_Subtract =	3,		//!< Subtract elements from selected set
	HIGHLIGHTMODE_Toggle =		4,		//!< Toggle selection status of elements
	};


// Colors for user interface elements.
const UINT32 UICOLOR_TextNormal =						0x00000000;
const UINT32 UICOLOR_TextDim =							0x00505050;		// Gray
const UINT32 UICOLOR_TextNotVisibleAtScale =			0x00208020;		// Dark Green
const UINT32 UICOLOR_TextNotInExtent =					0x00800000;		// Dark Blue
const UINT32 UICOLOR_TextNotYetRendered =				0x000000A0;		// Red
const UINT32 UICOLOR_TextEditable =						0x00800080;		// Purple
const UINT32 UICOLOR_TextExcludedFromGroupExtent =	0x000050A0;		// Dark Orange
const UINT32 UICOLOR_TextHasHighlightedElements =	0x00FF00FF;		// Magenta

//! View visibility bits and masks.
enum VISIBLEVIEW {
	VISIBLEVIEW_None =				0,
	VISIBLEVIEW_All =					0xFFFFFFFF,
	VISIBLEVIEW_AllNonLocator =	0xFFFFFFFE,
	VISIBLEVIEW_Locator =			0x00000001,
	VISIBLEVIEW_Hardcopy =			0x80000000,
	VISIBLEVIEW_AllNonHardcopy =	0x7FFFFFFF,
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(VISIBLEVIEW)
#endif

#define	MDISPVISIBLE_AllViews			GRE::VISIBLEVIEW_All

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_BASE_H
