/**
 * \file gre/uidlgmisc.h
 * \brief Miscellaneous User Interface dialogs
 *
 * \if NODOC
 * $Id: uidlgmisc.h_v 1.12 2013/02/26 17:35:26 mju Exp $
 *
 * $Log: uidlgmisc.h_v $
 * Revision 1.12  2013/02/26 17:35:26  mju
 * Minor format/dox issues.
 *
 * Revision 1.10  2009/12/11 15:30:44  mju
 * Change dlgRasterCorrelation to return errvalue.
 *
 * Revision 1.9  2009/12/11 14:39:10  mju
 * Add dlgRasterCorrelation.
 *
 * Revision 1.7  2008/07/09 14:36:09  ldyas
 * added DlgGoogleLock
 *
 * Revision 1.6  2008/05/27 16:08:51  mju
 * Add dlg3dViewPointUpdate.
 *
 * Revision 1.5  2008/03/06 17:40:16  mju
 * Add dlgLocatorCreate/Raise.
 *
 * Revision 1.4  2008/02/11 16:37:17  mju
 * Add dlgLayerCoordReport.
 *
 * Revision 1.3  2008/01/18 21:15:30  mju
 * Add dlgPrintViewSnapshot.
 *
 * Revision 1.2  2008/01/18 18:40:21  mju
 * Add dlgAdjust3dViewPoint.
 *
 * Revision 1.1  2008/01/11 18:41:02  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UIDLGMISC_H
#define	INC_GRE_UIDLGMISC_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace GRE {
	class LAYER;
	class LAYER_RASTER;
	class VIEW;
	class VIEWABLE;
	namespace UI {
		class VIEW_CANVAS;
		}
	}
#endif

namespace GRE {
namespace UI {

//=====================================================================================================================

//! Open (modeless) dialog for adjusting 3D viewpoint.
//! Defined in <gre/uidlgmisc.h>.
ERRVALUE DlgAdjust3dViewPointOpen (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	UI::VIEW_CANVAS *ViewCanvas
	);

//! Modeless dialog for viewing raw raster cell values.
//! Defined in <gre/uidlgmisc.h>.
ERRVALUE DlgExamineRawRaster (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	GRE::LAYER *layer
	);

void DlgGoogleLock (
	GRE::UI::VIEW_CANVAS *ViewCanvas,
	bool lock = true
	);

//! Dialog to report layer coordinates for pointer.
//! Defined in <gre/uidlgmisc.h>.
ERRVALUE DlgLayerCoordReport (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	GRE::LAYER *layer
	);

//! Create locator window for viewable.
//! Defined in <gre/uidlgmisc.h>.
void DlgLocatorCreate (
	GRE::VIEWABLE *viewable,
	bool useini = false					//!< Use INI setting to determine if should create or not
	);

//! Raise existing locator window for viewable.
//! Defined in <gre/uidlgmisc.h>.
void DlgLocatorRaise (
	GRE::VIEWABLE *viewable
	);

ERRVALUE DlgPrintViewSnapshot (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	GRE::VIEW *view
	);

//! Dialog to view raster correlation.
//! Defined in <gre/uidlgmisc.h>.
ERRVALUE DlgRasterCorrelation (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	GRE::LAYER_RASTER *layer		//!< Optional layer to start from
	);

//=====================================================================================================================

} // End namespace UI
} // End namespace GRE

#endif	// INC_GRE_UIDLGMISC_H
