/*
 * \file gre/uisurface.h
 * \brief GRE User Interface definitions for surface layer.
 *
 * \if NODOC
 * $Id: uisurface.h_v 1.1 2012/04/27 21:12:59 mju Exp $
 *
 * $Log: uisurface.h_v $
 * Revision 1.1  2012/04/27 21:12:59  mju
 * Initial revision
 *
 * \endif
 */

#ifndef	INC_GRE_UISURFACE_H
#define	INC_GRE_UISURFACE_H

#ifndef	INC_GRE_LRSURFACE_H
#include <gre/lrsurface.h>
#endif

#ifdef USE_MGUI

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif


namespace GRE {
//===================================================================================================================

//! Dialog for modifying layer display settings.
class LAYER_SURFACE::DISPPARM::DLG : public MGUI::DLGSHELL {
	public:

		//! Constructor.
		DLG (
			LAYER_SURFACE *pLayer,
			bool Include3D
			);

		//! Destructor.
		virtual ~DLG ();

		//! Get current LAYERDRAWFLAGS settings.
		LAYERDRAWFLAGS GetLayerDrawFlags () const;

		//! Get current DISPPARM values.
		const LAYER_SURFACE::DISPPARM& GetValues () const;

		//! Set DISPPARM and LAYERDRAWFLAGS values.
		ERRVALUE SetValues (
			const LAYER_SURFACE::DISPPARM& values,
			LAYERDRAWFLAGS LayerDrawFlags
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		LAYER_SURFACE *m_layer;

		void ApplyChanges (bool ForceRedraw);

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnApply ();
		virtual void v_OnClose ();
		virtual void v_OnOK ();
		virtual void v_OnOpen ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// USE_MGUI

#endif	// INC_GRE_UISURFACE_H
