/*
 * gre/uivector - GRE User Interface definitions for Vector layer
 *
 * \if NODOC
 * $Id: uivector.h_v 1.11 2011/10/20 20:26:24 mju Exp $
 *
 * $Log: uivector.h_v $
 * Revision 1.11  2011/10/20 20:26:24  mju
 * Remove editInTNTserver flag.
 *
 * Revision 1.10  2006/08/31 14:06:36  mju
 * Add onClose.
 *
 * Revision 1.9  2006/08/30 21:06:26  mju
 * Update for modeless.
 *
 * Revision 1.8  2006/04/13 16:26:53  dwilliss
 * doxygen problems
 *
 * Revision 1.7  2005/10/17 15:31:26  mju
 * Use gre namespace.
 *
 * Revision 1.6  2005/10/14 17:33:52  mju
 * GRE namespace and treeview.
 *
 * Revision 1.5  2003/12/15 21:51:18  mju
 * Use dlgshell.
 *
 * Revision 1.4  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.3  2003/07/30 15:44:35  mju
 * Ignore private section.
 *
 * Revision 1.2  2003/05/28 16:38:37  scowan
 * Removed definition of getobjfilter, replaced by geometric::isdisplayable.
 *
 * Revision 1.1  2002/09/19 17:09:36  mju
 * Initial revision
 *
 * \endif
*/

#ifndef	INC_GRE_UIVECTOR_H
#define	INC_GRE_UIVECTOR_H

#ifndef	INC_GRE_LRVECTOR_H
#include <gre/lrvector.h>
#endif

#ifdef USE_MGUI

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace GRE {
//===================================================================================================================
//! Dialog for modifying vector layer display settings.
class LAYER_VECTOR::DISPPARM::DLG : public MGUI::DLGSHELL {
	public:

		//! Constructor.
		DLG (
			LAYER_VECTOR *layer,				//!< Associated layer
			bool Include3D						//!< Include 3D settings
			);

		//! Destructor.
		virtual ~DLG (
			);

		//! Get 'mask' for which LAYERDRAWFLAG values are settable on dialog.
		LAYERDRAWFLAGS GetLayerDrawFlagMask (
			) const { return (LAYERDRAWFLAG_None); }

		//! Get current LAYERDRAWFLAGS settings.
		LAYERDRAWFLAGS GetLayerDrawFlags (
			) const;

		//! Get current DISPPARM values.
		const LAYER_VECTOR::DISPPARM& GetValues (
			) const;

		//! Set DISPPARM and LAYERDRAWFLAGS values.
		ERRVALUE SetValues (
			const LAYER_VECTOR::DISPPARM& values,
			LAYERDRAWFLAGS LayerDrawFlags
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		LAYER_VECTOR *m_layer;

		void ApplyChanges (bool ForceRedraw);
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnApply ();
		virtual void v_OnClose ();
		virtual void v_OnOK ();
		virtual void v_OnOpen ();

		friend class GRE::LAYER_VECTOR::DISPPARM::DLG::PRIV;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
}	// End namespace GRE

#endif	// USE_MGUI
#endif	// INC_GRE_UIVECTOR_H
