/**
 * \file mgui/dlgeditcolors.h
 * \brief MGUI::DLG_EDIT_COLORS class definitions.
 *
 * \if NODOC
 * $Id: dlgeditcolors.h_v 1.2 2010/06/02 18:00:49 mju Exp $
 *
 * $Log: dlgeditcolors.h_v $
 * Revision 1.2  2010/06/02 18:00:49  mju
 * Use color::capability enum.
 *
 * Revision 1.1  2007/10/31 15:05:03  mju
 * Initial revision
 *
 * Revision 1.9  2006/02/01 16:06:49  dwilliss
 * Added optional STYLE parameter
 *
 * Revision 1.8  2005/06/27 13:34:52  mju
 * Include edit hdr.
 *
 * Revision 1.7  2005/03/31 16:57:08  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.1  2002/10/23 15:39:25  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGEDITCOLORS_H
#define	INC_MGUI_DLGEDITCOLORS_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_FORMEDITCOLORS_H
#include <mgui/formeditcolors.h>
#endif

namespace MGUI {
//===================================================================================================================

//! Dialog for editing colors.
class DLG_EDIT_COLORS : public DLGSHELL {
	public:

		//! Constructor.
		//! Note that if transparency on/off or percentage controls are required for any
		//! item in the list the desired capability must be specified when constructing the dialog.
		explicit DLG_EDIT_COLORS (
			COLOR::CAPABILITY capability = COLOR::CAPABILITY_NoTransparency,
			LAYOUT_ORIENTATION orientation = LAYOUT_ORIENTATION_Vertical
			);

		//! Destructor.
		virtual ~DLG_EDIT_COLORS ();

		//! Add item to list.
		//! @return Item index for use when retrieve color.
		int AddItem (
			const MISTRING& name,
			const COLOR& color,
			COLOR::CAPABILITY capability = COLOR::CAPABILITY_AsPrevious
			) { return (m_FormEditColors.AddItem(name,color,capability)); }

		//! Add item to list.
		//! @return Item index for use when retrieve color.
		int AddItem (
			const MISTRING& name,
			UINT32 colorref,
			COLOR::CAPABILITY capability = COLOR::CAPABILITY_AsPrevious
			) { return (m_FormEditColors.AddItem(name,colorref,capability)); }

		//! Get color for specified item.
		//! @return Reference to color
		const COLOR& GetItemColor (
			int itemidx								//!< Item index (from AddItem())
			) { return (m_FormEditColors.GetItemColor(itemidx)); }

		//! Get color reference for specified item.
		//! @return Color reference
		UINT32 GetItemColorRef (
			int itemidx								//!< Item index (from AddItem())
			) { return (m_FormEditColors.GetItemColorRef(itemidx)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORM_EDIT_COLORS m_FormEditColors;
		COLOR::CAPABILITY m_capability;
		LAYOUT_ORIENTATION m_orientation;

		//! Overrides from DLGSHELL.
		virtual ERRVALUE v_CreateContent ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace MGUI

#endif	// INC_MGUI_DLGEDITCOLORS_H
