/**
 * \file mgui/formstereodevice.h
 * \brief MGUI::FORM_STEREODEVICE definitions.
 *
 * \if NODOC
 * $Id: formstereodevice.h_v 1.3 2012/03/01 16:36:59 mju Exp $
 *
 * $Log: formstereodevice.h_v $
 * Revision 1.3  2012/03/01 16:36:59  mju
 * Add style_NoOpenGL.
 *
 * Revision 1.2  2011/09/29 13:03:20  mju
 * Add create method without values parm.
 *
 * Revision 1.1  2011/09/28 21:22:49  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMSTEREODEVICE_H
#define	INC_MGUI_FORMSTEREODEVICE_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

class STEREODEVICE;

namespace MGUI {

//=====================================================================================================================
//! Form for specifying STEREODEVICE values.
class FORM_STEREODEVICE : public FORM_COMPOSITE {
	public:

		//! Form style.
		enum STYLE {
			STYLE_Default =			0x00,
			STYLE_NoAnaglyph =		0x01,	//!< Don't allow anaglyph choice
			STYLE_NoInterlace =		0x02,	//!< Don't allow interlace choices
			STYLE_NoSplit =			0x04,	//!< Don't allow "split window" choices
			STYLE_NoOpenGL =			0x08,	//!< Don't allow "OpenGL" choice if available
			STYLE_AllowSeparate =	0x10	//!< Allow "separate frames" choices with "separation" options
			};

		//! Default constructor.
		FORM_STEREODEVICE ();

		virtual ~FORM_STEREODEVICE ();

		//! Create form with previously specified or default values.
		void Create (
			LAYOUT_PANE_BASE& paneParent,		//!< Parent pane
			STYLE style = STYLE_Default		//!< Form style
			);

		//! Create form with specified initial values.
		void Create (
			LAYOUT_PANE_BASE& paneParent,		//!< Parent pane
			const STEREODEVICE& values,		//!< Initial values
			STYLE style = STYLE_Default		//!< Form style
			);

		//! Get current values.
		const STEREODEVICE& GetValues () const;

		//! Set delegate to call when any setting is changed.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set values.
		void SetValues (
			const STEREODEVICE& values,			//!< New values to set
			bool notify								//!< Notify via delegate if changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMSTEREODEVICE_H
