/**
 * \file mgui/toolgps.h
 * \brief MGUI::TOOL_GPS class definitions.
 *
 * \if NODOC
 * $Id: toolgps.h_v 1.2 2011/12/09 20:23:05 mju Exp $
 *
 * $Log: toolgps.h_v $
 * Revision 1.2  2011/12/09 20:23:05  mju
 * No context in v_draw.
 *
 * Revision 1.1  2008/04/28 23:04:30  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TOOLGPS_H
#define	INC_MGUI_TOOLGPS_H

#ifndef	INC_MGUI_TOOLBASE_H
	#include <mgui/toolbase.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace MGPS {
	class SOURCE;
	struct REPORT;
	}
#endif // GENERATING_DOXYGEN_OUTPUT

namespace MGUI {

//! Class for derived graphic tools that support GPS devices or logs
class TOOL_GPS : public TOOL_BASE {
	public:

		//! Destructor
		virtual ~TOOL_GPS () = 0;

		//! Create the GPS form with the GPS selection and operational controls.  
		//! If you want to place the icons in your own CTRL_TOOLBAR, call this 
		//! method before the tool's v_CreateControlForm() is called.
		//! TOOL_BASE::SetTransToMap() MUST be set for this to work.
		MGUI::FORM* CreateGPSForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane	//!< Parent pane
			);

	protected:

		//! Constructor
		explicit TOOL_GPS (
			TOOLCANVAS *canvas,
			CONTROLNEED ControlNeed = CONTROLNEED_None,	//!< Need for control dialog or form
			CAPABILITY capability = CAPABILITY_Default	//!< Additional capabilities
			);
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef TOOL_BASE BASECLASS;

		class CONTROLFORM;
		friend class CONTROLFORM;
		class SOURCEOBSERVER;
		friend class SOURCEOBSERVER;
		class MANAGEROBSERVER;
		friend class MANAGEROBSERVER;
	
		MGPS::SOURCE* m_GPSsource;
		SOURCEOBSERVER* m_SourceObserver;
		MANAGEROBSERVER* m_ManagerObserver;
		CONTROLFORM* m_ControlForm;
		TRANS2D_MAPGEN m_GPStoTool;
		DPOINT3D m_CurrentGPSPoint;
		bool m_UseGPSCoord;
		
		void SendGPSCoordToTool (const MGPS::REPORT& gpsdata);

		void v_OnSetTransToMap ();
		#endif 		// GENERATING_DOXYGEN_OUTPUT

		//! Called when GPS coordinate is obtained either through the GPS source timeout
		//! or the user pressing the "Track GPS" button.
		virtual void v_OnGPSCoordinate (
			const DPOINT3D& point				//!< GPS point sent in Tool coordinates
			) = 0;
	};
	
}	// End namespace MGUI

#endif	// INC_MGUI_TOOLGPS_H

