/**
 * \file mi32/inibase.h
 * \brief Low-level Ini file definitions
 *
 * \if NODOC
 * $Id: inibase.h_v 1.1 2006/09/25 16:41:11 mju Exp $
 *
 * $Log: inibase.h_v $
 * Revision 1.1  2006/09/25 16:41:11  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_INIBASE_H
#define	INC_MI32_INIBASE_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif


#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif


#define INITYPE_IniStruct		0x01000000					//!< Ini structure marker
#define INITYPE_End				0								//!< End of a list (0)
#define INITYPE_ByteBoolean	1								//!< Read a Yes/No, 1/0, Y/N, y/n, True/False, T/F, t/f
#define INITYPE_IntBoolean		2								//!< Read a Yes/No, 1/0, Y/N, y/n, True/False, T/F, t/f
#define INITYPE_INT8				3								//!< Read a byte
#define INITYPE_UINT8			4								//!< Read a unsigned byte
#define INITYPE_INT16			5								//!< Read a word
#define INITYPE_UINT16			6								//!< Read a unsigned word
#define INITYPE_INT32			7								//!< Read a long
#define INITYPE_UINT32			8								//!< Read a unsigned long
#define INITYPE_Float			9								//!< Read a float
#define INITYPE_Double			10								//!< Read a double
#define INITYPE_String			11								//!< Read a string
#define INITYPE_ASCII			INITYPE_String				//!< Read a string
#define INITYPE_Unicode			12								//!< Read a MIUNICODE string
#define INITYPE_MIUNICODE		INITYPE_Unicode
#define INITYPE_DPOINT2D		13|INITYPE_IniStruct		//!< Read a 2D point
#define INITYPE_DPOINT3D		14|INITYPE_IniStruct		//!< Read a 3D point
#define INITYPE_COLOR			17|INITYPE_IniStruct		//!< Read a COLOR
#define INITYPE_DRECT2D			18|INITYPE_IniStruct		//!< Read a DRECT2D
#define INITYPE_Bool				19								//!< bool

extern "C" {

#ifndef GENERATING_DOXYGEN_OUTPUT
// Internal functions

LIBEXPORT INT32 _iniRead (
	INIHANDLE iniptr, 						//!< Open INI file handle, NULL for default .ini file		
	const char* group, 						//!< Group to find field in, uses GetAppName() if NULL
	const char*const field, 				//!< Field that the data is stored under
	UINT32 initype, 							//!< INITYPE of items to read
	void *data, 								//!< Pointer to buffer to read into, dependant on 'initype'
	INT32 count = 1							//!< Number of items to read
	);

LIBEXPORT INT32 _iniReadString (
	INIHANDLE iniptr, 						//!< Open INI file handle, NULL for default
	const char* group, 						//!< Group to find field in, uses GetAppName() if NULL
	const char*const field, 				//!< Field that data is stored in
	UINT32 initype, 							//!< INITYPE_ASCII or INITYPE_Unicode
	void *data, 								//!< Pointer to buffer to read into RETURNED
	INT32 len									//!< Number of BYTES to read
	);

LIBEXPORT INT32 _iniReadStringAllocML (
	INIHANDLE iniptr, 						//!< Open INI file handle, NULL for default
	const char* group, 						//!< Group to find field in, uses GetAppName() if NULL
	const char*const field, 				//!< Field that data is stored in
	UINT32 initype, 							//!< INITYPE_ASCII or INITYPE_Unicode
	void **data									//!< Pointer to buffer to read into RETURNED
	);

LIBEXPORT int _iniWrite (
	INIHANDLE iniptr, 						//!< Ini file handle, NULL for default .ini file
	const char* group, 						//!< Group to find field in, uses GetAppName() if NULL
	const char* field, 						//!< Field that the data is stored under
	UINT32 initype, 							//!< Type of data to write
	const void *data, 						//!< Pointer to buffer to write from
	INT32	count									//!< Number of entries to write
	);

LIBEXPORT int _iniWriteString (
	INIHANDLE iniptr, 						//!< Open INI file handle NULL for default
	const char *group, 						//!< Group to locate field in, uses GetAppName() if NULL
	const char *field, 						//!< Field that the data is to be stored under
	UINT32 initype, 							//!< INITYPE_ASCII or INITYPE_Unicode
	const void *data							//!< Pointer to buffer to write from
	);

LIBEXPORT int _iniWriteStringML (
	INIHANDLE iniptr, 						//!< Open INI file handle NULL for default
	const char *group, 						//!< Group to locate field in, uses GetAppName() if NULL
	const char *field, 						//!< Field that the data is to be stored under
	UINT32 initype, 							//!< Type of data to write
	const void *data							//!< Pointer to buffer to write from
	);

}	// Extern "C"

#endif // GENERATING_DOXYGEN_OUTPUT

#undef	LIBEXPORT

#endif	// INC_MI32_INIBASE_H
