/******************************************************************************
 *
 * \file mi32/mg2mem24.h
 * \brief A RAM-based implementation of MGD::DEVICE for 24-bit RGB visuals
 *
 * \if NODOC
 * $Log: mg2mem24.h_v $
 * Revision 1.18  2011/10/26 17:57:57  mju
 * Frame buffer support.
 *
 * Revision 1.17  2005/09/15 21:39:30  mju
 * Fix unref parm warning.
 *
 * Revision 1.16  2005/01/26 18:07:42  mju
 * Remove scanColorUse.
 *
 * Revision 1.15  2004/06/02 15:08:02  mju
 * Add COLORORDER enum and require in create().
 *
 * Revision 1.14  2003/12/15 23:36:21  dwilliss
 * Diverging for 7.0
 * Made changes necessary for graphical data tips
 *
 * Revision 1.13  2003/12/10 23:48:08  dwilliss
 * cleanup
 *
 * Revision 1.12  2003/10/01 22:16:37  dwilliss
 * Doxygen
 *
 * Revision 1.11  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.10  2003/06/11 17:12:22  dwilliss
 * Virtual methods all renamed to start with v_ and made private (where
 * possible) or protected.
 *
 * Revision 1.9  2002/10/17 15:22:22  mju
 * COLORSPACE enum values changed.
 *
 * Revision 1.8  2002/01/09 16:57:51  dwilliss
 * FillSpanDirect needed const
 *
 * Revision 1.3  2001/07/27 22:14:50  dwilliss
 * Set capability flag for mask in image span
 *
 * Revision 1.1  2001/07/11 17:33:44  dwilliss
 * Initial revision
 *
 * \endif
 */
#ifndef INC_MI32_MG2MEM24_H
#define INC_MI32_MG2MEM24_H

#ifndef INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif


namespace MGD {


class CONTEXT_MEM24;

class DEVICE_MEM24 : public MGD::DEVICE {

	public:

		enum COLORORDER {
			COLORORDER_RGB,
			COLORORDER_BGR
			};

		//! constructor
		DEVICE_MEM24 ();

		//! Destructor
		virtual ~DEVICE_MEM24 ();

		ERRVALUE Create (
			INT32 height,					//!< Height in pixels
			INT32 width,					//!< Width in pixels
			COLORORDER colororder		//!< Order of color bytes
			);

		CAPABILITY GetCapabilities (
			) const {
			return (v_GetCapabilities());
			}

	protected:

		//! Fill a horizontal span.
		virtual ERRVALUE v_FillSpan (
			MGD::CONTEXT* vgc,
			INT32 x1,
			INT32 x2,
			INT32 y
			);

		virtual CAPABILITY v_GetCapabilities () const;

		UINT32 m_BytesPerLine;
		bool m_bIsRGB;	//!< if false, data is BGR

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		// Unimplemented
		DEVICE_MEM24 (const DEVICE_MEM24&);

		UINT8 *m_buf;

		friend class MGD::CONTEXT_MEM24;

		virtual CONTEXT* v_CreateContext ();
		virtual UINT8* v_GetImagePointer (INT32 x, INT32 y, INT32 len, UINT16 FrameBuffer);
		virtual COLORSPACE v_GetColorSpace () const;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

		
class CONTEXT_MEM24 : public MGD::CONTEXT {
	public:

		virtual ~CONTEXT_MEM24();

	protected:

		UINT8 m_pixbyte0;
		UINT8 m_pixbyte1;
		UINT8 m_pixbyte2;

		UINT8 m_maskbyte0;
		UINT8 m_maskbyte1;
		UINT8 m_maskbyte2;

		//! Default Constructor 
		CONTEXT_MEM24 (
			MGD::DEVICE_MEM24* disp
			);

		void v_UpdateColor();
		void v_UpdatePlaneMask();

		virtual ERRVALUE v_PutImageSpanDirect (
			INT32 x,
			INT32 y,
			INT32 len,
			const void* data,
			const UINT8* mask
			);

		virtual ERRVALUE v_PutPixel (
			INT32 x,
			INT32 y,
			MGD::PIXEL pixel
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		MGD::DEVICE_MEM24* m_DevMem24;

		//! Copy constructor private and unimplemented
		CONTEXT_MEM24(const CONTEXT_MEM24&);
	
		friend class MGD::DEVICE_MEM24;	// Only MGD_DEVICE::CreateGC() can call our constructor
		#endif // GENERATING_DOXYGEN_OUTPUT



	};


}	//! End of namespace

#endif	// INC_MI32_MG2MEM24_H

