/**
 * \file rgngen.h <mi32/rgngen.h>
 * \brief Region generation functions and structures
 *
 * \if NODOC
 * $Id: rgngen.h_v 1.18 2008/01/09 14:55:35 mju Exp $
 *
 * $Log: rgngen.h_v $
 * Revision 1.18  2008/01/09 14:55:35  mju
 * Add regiongenmethod enum and use for regiongen defines.
 *
 * Revision 1.17  2007/02/09 13:49:20  mju
 * Fwd declare rvc::objitem.
 *
 * Revision 1.16  2006/09/28 17:11:39  mju
 * Remove floodhist.
 *
 * Revision 1.15  2006/09/12 16:01:37  scowan
 * Basin function now takes objitem.
 *
 * Revision 1.14  2006/09/11 21:57:55  scowan
 * Another region function now takes a objitem.
 *
 * Revision 1.13  2006/09/11 19:35:03  scowan
 * Removed passing vector object handle in some functions.
 *
 * Revision 1.12  2006/09/11 17:17:20  scowan
 * Chagned voronoi to region prototype.
 *
 * Revision 1.11  2004/07/20 14:05:25  scowan
 * Removed region from vector polygon prototype.
 *
 * Revision 1.10  2004/07/16 15:27:14  scowan
 * Added forward declaration.
 *
 * Revision 1.9  2004/07/15 19:28:31  mju
 * Use region2d for all region generation functions.
 *
 * Revision 1.8  2004/07/14 23:14:43  scowan
 * Removed prototypes from archived source module.
 *
 * Revision 1.7  2004/07/14 21:50:34  dwilliss
 * removed prototype for a function that's no longer public
 *
 * Revision 1.6  2004/07/02 21:32:36  scowan
 * Removed cad region prototype.
 *
 * Revision 1.5  2004/07/02 17:07:33  dwilliss
 * Marked some things as deprecated
 *
 * Revision 1.4  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.3  2003/08/05 20:48:43  vdronov
 * remove buffzone.h
 *
 * Revision 1.2  2002/03/20 16:06:51  vdronov
 * mofidied MregionFromBufferZone
 *
 * Revision 1.1  2000/11/15 15:02:55  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_RGNGEN_H
#define	INC_MI32_RGNGEN_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class REGION2D;		// Forward declaration
namespace RVC {
	class OBJITEM;
	}
#endif // GENERATING_DOXYGEN_OUTPUT


//! Region generation method enumeration.
enum REGIONGENMETHOD {
	REGIONGENMETHOD_SelectedElements =	1,
	REGIONGENMETHOD_PolygonFitting =		2,
	REGIONGENMETHOD_BufferZone =			3,
	REGIONGENMETHOD_Viewshed =				4,
	REGIONGENMETHOD_Threshold =			5,
	REGIONGENMETHOD_Voronoi =				6,
	REGIONGENMETHOD_Basin =					7,
	REGIONGENMETHOD_FloodZone =			8,
	REGIONGENMETHOD_RasterCells =			9,
	REGIONGENMETHOD_TextureGrowth = 		10,
	REGIONGENMETHOD_Cluster = 				11,
	REGIONGENMETHOD_RasterExpression = 	12,
	REGIONGENMETHOD_RasterTraceSolid =	13,
	REGIONGENMETHOD_RasterTraceBound =	14,
	};


#define REGIONGEN_SelPolys				REGIONGENMETHOD_SelectedElements
#define REGIONGEN_PolyFit				REGIONGENMETHOD_PolygonFitting
#define REGIONGEN_BufZone				REGIONGENMETHOD_BufferZone
#define REGIONGEN_ViewShed				REGIONGENMETHOD_Viewshed
#define REGIONGEN_Threshold			REGIONGENMETHOD_Threshold
#define REGIONGEN_Voronoi				REGIONGENMETHOD_Voronoi
#define REGIONGEN_Basin					REGIONGENMETHOD_Basin
#define REGIONGEN_FloodZone			REGIONGENMETHOD_FloodZone
#define REGIONGEN_RasterCells			REGIONGENMETHOD_RasterCells
#define REGIONGEN_TextureGrowth 		REGIONGENMETHOD_TextureGrowth 
#define REGIONGEN_Cluster 				REGIONGENMETHOD_Cluster 
#define REGIONGEN_RasterExpression 	REGIONGENMETHOD_RasterExpression 
#define REGIONGEN_RasterTraceSolid	REGIONGENMETHOD_RasterTraceSolid
#define REGIONGEN_RasterTraceBound	REGIONGENMETHOD_RasterTraceBound






/***
*	Structure and defines for polygon fitting functions to a region
***/

#define MP_sign_default    TRUE

#define MP_iter_min        2
#define MP_iter_default    5
#define MP_iter_max        100

#define MP_weight_min      0.010
#define MP_weight_default  0.618
#define MP_weight_max      1.000

#define MP_factor_min      0.01
#define MP_factor_default  0.50
#define MP_factor_max      1.00


#define HM_sign_default    TRUE

#define HM_iter_min        2
#define HM_iter_default    5
#define HM_iter_max        100

#define HM_weight_min      0.010
#define HM_weight_default  0.618
#define HM_weight_max      1.000

#define HM_horiz_min       15
#define HM_horiz_default   15
#define HM_horiz_max       120

#define HM_vert_min        15
#define HM_vert_default    15
#define HM_vert_max        120

#define HM_percent_min     10.00
#define HM_percent_default 95.00
#define HM_percent_max     100.0


#define FT_horiz_min       15
#define FT_horiz_default   15
#define FT_horiz_max       120

#define FT_vert_min        15
#define FT_vert_default    15
#define FT_vert_max        120

#define FT_height_min      0.01
#define FT_height_default  0.50
#define FT_height_max      1.00


#define TS_sign_default    TRUE

#define TS_iter_min        2
#define TS_iter_default    5
#define TS_iter_max        100

#define TS_weight_min      0.010
#define TS_weight_default  0.618
#define TS_weight_max      1.000

#define TS_percent_min     10.00
#define TS_percent_default 95.00
#define TS_percent_max     100.0

#define MC_sign_default    TRUE

#define MC_iter_min        2
#define MC_iter_default    5
#define MC_iter_max        100

#define MC_weight_min      0.010
#define MC_weight_default  0.618
#define MC_weight_max      1.000

#define MC_percent_min     10.00
#define MC_percent_default 95.00
#define MC_percent_max     100.0

#define KN_mode_fixed      0
#define KN_mode_adaptive   1

#define KN_grid_manual     0
#define KN_grid_simple     1
#define KN_grid_lscv			2

#define KN_horiz_min       4
#define KN_horiz_default   15
#define KN_horiz_max       120

#define KN_vert_min        4
#define KN_vert_default    15
#define KN_vert_max        120

#define KN_height_min      0.001
#define KN_height_default  0.50
#define KN_height_max      1.00


struct MINPOLYPARM {
   INT32   sign;								//!<  use or not eliminate outliers 
   UINT32  iter;					      	//!<  number of iteration in removing outliers 
   double weight;								//!<  threshold for removing outliers 
	double factor;								//!<  from 0 to 1. 1 = concave poly 
   };

struct HARMONPARM {
	INT32   sign;								//!<  use or not eliminate outliers 
   UINT32  iter;		      				//!<  number of iteration in removing outliers 
   double weight; 							//!<  threshold for removing outliers 
	UINT32  horiz;     						//!<  number of horizontal/vertical grids 
	UINT32  vert;
	double percent;  					    	//!<  % of points to include 
	};

struct FOURIERPARM {
	UINT32 horiz;
	UINT32 vert;
	double height;								//!<  slice height in computing fourier 
	};

struct TESSELPARM {
	INT32  sign;								//!<  use or not eliminate outliers 
  	UINT32 iter;      						//!<  number of iteration in removing outliers 
   double weight;								//!<  threshold for removing outliers 
	double percent;
	};

struct MINCONVEXPARM {
	INT32  sign;								//!<  use or not eliminate outliers 
  	UINT32 iter;      						//!<  number of iteration in removing outliers 
   double weight;								//!<  threshold for removing outliers 
	double percent;
	};

struct KERNELPARM {
	INT32 mode;
	INT32 grid;
	UINT32 horiz;
	UINT32 vert;
	double height;								//!<  slice height in computing fourier 
	};

struct POLYFITPARM {
	UINT32 method;
	MINPOLYPARM minpoly;
	HARMONPARM  harmon;
	FOURIERPARM fourier;
	TESSELPARM  tessel;
	MINCONVEXPARM minconvex;
	KERNELPARM kernel;
	};


#define	RGNPFIT_MINPOLY 			1
#define	RGNPFIT_HARMON  			2
#define	RGNPFIT_FOURIER 			3
#define	RGNPFIT_TESSEL  			4
#define	RGNPFIT_MINCONVEX  		5
#define	RGNPFIT_KERNEL		  		6
#define	RGNPFIT_COUNT  			7

#define		CLMETHOD_K_means            	0
#define		CLMETHOD_FC_means           	1
#define		CLMETHOD_MinDistAngle       	2
#define		CLMETHOD_SelfOrganization   	3
#define		CLMETHOD_AdaptiveResonance  	4
#define		CLMETHOD_Isodata            	5
#define		CLMETHOD_COUNT						6             

struct KMEANS_PARM {
	DOUBLE MaxDistPercent;
	DOUBLE MaxMoveSteady;
	DOUBLE MinSteadyPct;
	INT32	 MaxIterations;
   };

struct FCMEANS_PARM {
	DOUBLE MaxDistPercent;
	INT32	 MaxIterations;
   };

struct MDANGLE_PARM {
	DOUBLE MaxAnglePercent;
	DOUBLE XPosPercent;
	DOUBLE YPosPercent;
	DOUBLE MaxMoveSteady;
	DOUBLE MinSteadyPct;
	INT32	 MaxIterations;
   };

struct SELFORG_PARM {
	DOUBLE MaxDistPercent;
	DOUBLE MaxMoveSteady;
	DOUBLE MinSteadyPct;
	INT32	 MaxIterations;
   };

struct ADRES_PARM {
	DOUBLE MaxDistPercent;
	INT32	 MaxIterations;
   };

struct ISODATA_PARM {
	INT32 ReqClasses;
	INT32 MaxIterations;
	INT32 MinClusterCells;
	DOUBLE SplitMaxStdDev;
	DOUBLE CombineMinDist;
	DOUBLE ChainMinDist;
   };

struct CLASSIFY_PARM {
	UINT32 method;
	KMEANS_PARM kmeans;
	FCMEANS_PARM fcmeans;
	MDANGLE_PARM mdangle;
	SELFORG_PARM selforg;
	ADRES_PARM adres;
	ISODATA_PARM isodata;
   };

struct RGNELEMLIST {
	int ElemType;
	UINT8 *set;
	UINT32 NumElem;
	UINT32 NumSelected;
	};

extern "C" {

//!:Associate with "Region Generation Functions"
//!\addtogroup RegionGen Region Generation Functions
//!@{

//! Compute region from set of points using polygon fitting.
int ComputeHomerange (
	DPOINT2D **Points, 
	INT32 NumPoints, 
	DPOLYGON **Dpolys, 
	INT32 *NumDpolys, 
	POLYFITPARM *pfparm
	);

//! Create a region from basin.
int MregionFromBasin (
	const RVC::OBJITEM& SrcRastObjItem,
	const RVC::OBJITEM& SrcVectObjItem,
	int NumElemTypes,
	RGNELEMLIST *ElemList,
	REGION2D& region
	);

//! Create a region from buffer zone.
int MregionFromBufferZone (
	const RVC::OBJITEM& SrcObjItem,
	int NumElemTypes,
	RGNELEMLIST *ElemList,
	DOUBLE PointDistance,
	DOUBLE LineDistance,
	DOUBLE PolyDistance,
	int Unit,
	bool side,
	REGION2D& region
	);

//! Create a region from a cluster.
ERRVALUE MregionFromCluster (
	const RVC::OBJITEM& SrcGeomObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes, 						//!< Number of element types
	CLASSIFY_PARM *clparm, 
	REGION2D& region
	);


//! Generate a region from a polygon fit of selected points.
//! 
//! @return 1 if one or more regions generated, 0 if no regions generated, error code if < 0.
//!
//! Polygon fit methods:			
//!		\li\b RGNPFIT_MINPOLY		
//!		\li\b RGNPFIT_HARMON		
//!		\li\b RGNPFIT_FOURIER
//!		\li\b RGNPFIT_TESSEL
int MregionFromPolygonFit (
	const RVC::OBJITEM& SrcGeomObjItem,
	RGNELEMLIST *bitarray,					//!< Selected points to generate polygon around
	int NumPoints,								//!< Number of points that the array covers
	POLYFITPARM *pfparm,						//!< Polygon fit parameter structure
	REGION2D& region
	);

//! Create a region by raster texture growing.
int MregionFromTextureGrowth (
	int rhandle, 
	int vhandle,
	int NumElemTypes,
	RGNELEMLIST *ElemList,
	DOUBLE numSD, 
	INT32 Around, 
	REGION2D& region,
	UINT32 ProcessType
	);

//! Create a region from Voronoi diagram.
ERRVALUE MregionFromVoronoi (
	const RVC::OBJITEM& SrcGeomObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,							//!< Number of element types
	DOUBLE ExtFactor,
	REGION2D& region							//!< Region returned
	);

//! Initialize parameters to defaults for polygon fitting.
//!
//! See MregionFromPolygonFit() for details on POLYFITPARM and methods.
void MregionInitPolygonFit (
	POLYFITPARM *pfparm, 					
	UINT32 method
	);

//!@}

}	//! extern "C"

#endif	//!< INC_MI32_RGNGEN_H
