/**
 * \file mi32/xtargs.h
 * \brief XTARGS definition
 *
 * \if NODOC
 * $Id: xtargs.h_v 1.1 2011/08/19 14:01:23 mju Exp $
 *
 * $Log: xtargs.h_v $
 * Revision 1.1  2011/08/19 14:01:23  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_XTARGS_H
#define	INC_MI32_XTARGS_H

#ifndef	_XtIntrinsic_H_
	#include <X11/Intrinsic.h>
#endif

//------------------------------------------------------------------------------

//! Class to contain XtArg array.
class XTARGS {
	public:

		//! Default constructor, creates empty array.
		XTARGS (
			): m_num(0) { }

		//! Construct from Arg array and count.
		XTARGS (
			const Arg *arg,							//!< Argument array
			int count									//!< Number of items to copy from
			): m_num(count) { memcpy(m_arg,arg,count*sizeof(arg[0])); }

		//! Constant cast to Arg*.
		operator const Arg*(
			) const { return (m_arg); }

		//! Non-constant cast to Arg*.
		operator Arg*(
			) { return (m_arg); }

		//! Read (const) access to array element.
		//! @note Does not verify that index is within valid range.
		const Arg& operator[] (
			int index
			) const { return (m_arg[index]); }
#ifndef GENERATING_DOXYGEN_OUTPUT
/*
		//! Append item to array.
		template <class _T> void Append (
			const char *name,							//!< Argument name
			_T value										//!< Value
			) { m_arg[m_num].name = const_cast<char*>(name); m_arg[m_num++].value = (XtArgVal)value; }
*/
#endif //!< GENERATING_DOXYGEN_OUTPUT
		//! Append item to array.
		void Append (
			const char *name,							//!< Argument name
			void *value									//!< Value
			) { m_arg[m_num].name = const_cast<char*>(name); m_arg[m_num++].value = (XtArgVal)value; }

		//! Append item to array.
		void Append (
			const char *name,							//!< Argument name
			int value									//!< Value
			) { m_arg[m_num].name = const_cast<char*>(name); m_arg[m_num++].value = (XtArgVal)value; }

		//! Append Arg to array.
		void Append (
			const Arg& arg
			) { m_arg[m_num++] = arg; }

		//! Assign from Arg array and count.
		void Assign (
			const Arg *arg,							//!< Argument array
			int count									//!< Number of items to copy from
			) { m_num = count; memcpy(m_arg,arg,count*sizeof(arg[0])); }

		//! Clear argument list.
		void Clear (
			) { m_num = 0; }

		//! Delete item at specified index.
		void DeleteItem (
			int idx
			);

		//! Find item in array by name.
		//! @return Index to item or -1 if not found.
		int FindItem (
			const char *name
			) const;

		//! Add form attachment parameters for bottom.
		void FormAttachBottom (
			Widget widget = 0,						//!< Widget to attach to, 0 for parent form itself
			int offset = -1,							//!< Offset, -1 to use parent form spacing
			bool opposite = false					//!< 'true' for opposite-side attachment
			);

		//! Add form attachment parameters for bottom by position.
		void FormAttachBottomPosition (
			int position,								//!< Position relative to fraction base
			int offset = -1							//!< Offset, -1 to use parent form spacing
			);

		//! Add form attachment parameters for left.
		void FormAttachLeft (
			Widget widget = 0,						//!< Widget to attach to, 0 for parent form itself
			int offset = -1,							//!< Offset, -1 to use parent form spacing
			bool opposite = false					//!< 'true' for opposite-side attachment
			);

		//! Add form attachment parameters for left by position.
		void FormAttachLeftPosition (
			int position,								//!< Position relative to fraction base
			int offset = -1							//!< Offset, -1 to use parent form spacing
			);

		//! Add form attachment parameters for right.
		void FormAttachRight (
			Widget widget = 0,						//!< Widget to attach to, 0 for parent form itself
			int offset = -1,							//!< Offset, -1 to use parent form spacing
			bool opposite = false					//!< 'true' for opposite-side attachment
			);

		//! Add form attachment parameters for right by position.
		void FormAttachRightPosition (
			int position,								//!< Position relative to fraction base
			int offset = -1							//!< Offset, -1 to use parent form spacing
			);

		//! Add form attachment parameters for top.
		void FormAttachTop (
			Widget widget = 0,						//!< Widget to attach to, 0 for parent form itself
			int offset = -1,							//!< Offset, -1 to use parent form spacing
			bool opposite = false					//!< 'true' for opposite-side attachment
			);

		//! Add form attachment parameters for top by position.
		void FormAttachTopPosition (
			int position,								//!< Position relative to fraction base
			int offset = -1							//!< Offset, -1 to use parent form spacing
			);

		//! Get XtArg array.
		const Arg* GetArray (
			) const { return (m_arg); }

		//! Get XtArg array.
		Arg* GetArray (
			) { return (m_arg); }

		//! Get number of items in array.
		int GetCount (
			) const { return (m_num); }

		//! Get item from array.
		const Arg& GetItem (
			int idx
			) const { return (m_arg[idx]); }

		//! Get values from widget.
		void GetWidgetValues (
			Widget widget
			) { XtGetValues(widget,m_arg,m_num); }

		//! Set values to widget.
		void SetWidgetValues (
			Widget widget
			) { XtSetValues(widget,m_arg,m_num); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		Arg m_arg[30];
		int m_num;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//------------------------------------------------------------------------------

#endif	// INC_MI32_XTARGS_H
