/**
*	\file <rvc/objitemlist.h>
*	\brief RVCOBJITEMLIST defns
*
*	\if NODOC
*	$Id: objitemlist.h_v 1.3 2011/01/14 21:16:49 mju Exp $
*
*	$Log: objitemlist.h_v $
*	Revision 1.3  2011/01/14 21:16:49  mju
*	Add iniRead/Write.
*
*	Revision 1.2  2007/08/16 22:01:21  mju
*	Add objitemlist methods for getDisplay/ToolTipString and isAllSameFile.
*
*	Revision 1.1  2007/05/08 12:57:36  mju
*	Initial revision
*
*	\endif
**/

#ifndef	INC_RVC_OBJITEMLIST_H
#define	INC_RVC_OBJITEMLIST_H

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif


#ifdef RVCSYSDLL
	#define CLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace RVC {

//! List of OBJITEMs.
class CLASSEXPORT OBJITEMLIST : public MILIST<OBJITEM> {
	public:

		//! Get standard single-line "display" string for list.
		MISTRING GetDisplayString (
			int MaxItemsListed,							//! Maximum number of items to list, will show ellipsis (...) if more
			const CREDENTIALS* pCredentials = 0
			) const;

		//! Get standard multi-line "tooltip" string for list.
		//! Will use at most two lines per item.
		MISTRING GetToolTipString (
			int MaxItemsListed,							//! Maximum number of items to list, will show ellipsis (...) if more
			const CREDENTIALS* pCredentials = 0
			) const;

		//! Read from .ini file.
		//! List will be cleared prior to reading.
		void IniRead (
			INIHANDLE handle,							//!< Ini Handle created by IniOpen()
			const char *group,						//!< Group for ini entry, App Name if 0
			const char *field							//!< Field to read string from
			);

		//! Write to .ini file.
		void IniWrite (
			INIHANDLE handle,							//!< Ini Handle created by IniOpen()
			const char *group,						//!< Group to write ini entry, App Name if 0
			const char *field,						//!< Field to write string to
			const CREDENTIALS *pCredentials = 0
			) const;

		//! Determine if all items in list are in same file
		bool IsAllSameFile (
			) const;
	};

}		// End namespace RVC

#undef	CLASSEXPORT
#endif	// INC_RVC_OBJITEMLIST_H
