/**
 * \file rvc/tiept.h 
 * \brief interface for the RVC::TIEPOINT class.
 *
 * \if NODOC
 * $Id: tiept.h_v 1.12 2011/12/23 14:15:46 mju Exp $
 *
 * $Log: tiept.h_v $
 * Revision 1.12  2011/12/23 14:15:46  mju
 * Remove redundant hdr.
 *
 * Revision 1.11  2007/09/06 17:19:05  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.10  2005/12/09 17:30:01  scowan
 * Updated to use credentials.
 *
 * Revision 1.1  2000/09/29 20:19:04  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_TIEPT_H
#define INC_RVC_TIEPT_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
struct TIEPOINT;
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_TiePoint objects.  
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_TiePoint.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Raster
//!
//!	Read() and Write() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods
class TIEPOINTOBJ : public RVC::SIMPLE {
	public:
		TIEPOINTOBJ (
			);

		TIEPOINTOBJ (
			const RVC::TIEPOINTOBJ& rhs
			);

		virtual ~TIEPOINTOBJ (
			);

		RVC::TIEPOINTOBJ& operator= (
			const RVC::TIEPOINTOBJ& rhs
			);

		//! Get the Match ID for the tie point rvc object
		//! @return The match ID
		UINT32 GetMatchID (
			) const;

		//! Obtain the tie point list from the instance.  The rvc object must have been Read() first
		//! @return Constant array of tie points.
		const SIMPLE_ARRAY<TIEPOINT>& GetPointList (
			) const;

		//! Sets the match ID
		void SetMatchID (
			UINT32 MatchID
			);

		//! Sets tie point list in this instance.  Used for writing
		ERRVALUE SetPointList (
			const SIMPLE_ARRAY<TIEPOINT>& PointList		//!< Tie point list to set instance to
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
			
		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		SIMPLE_ARRAY<TIEPOINT> m_PointList;
		UINT32 m_MatchID;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif 		// INC_RVC_TIEPT_H
