/******************************************************************************
 *
 * \file sml/stype.h
 * \brief SML::STYPE enum -- Symbol types
 *
 * \if NODOC
 * $Log: stype.h_v $
 * Revision 1.5  2007/05/08 15:46:24  dwilliss
 * SML namespace
 *
 * Revision 1.4  2007/05/04 22:57:11  dwilliss
 * moved stypeflags here
 *
 * Revision 1.3  2007/05/04 21:32:27  dwilliss
 * moved some stuff out of sml.h
 *
 * Revision 1.2  2007/05/04 20:28:03  dwilliss
 * comments
 *
 * Revision 1.1  2007/05/04 20:27:21  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_STYPE_H
#define INC_SML_STYPE_H

/***************************************************************************/
/*
 *  Symbol types
 */

//! This was not moved into the SML namespace because doing so would break
//! a lot of existing code
enum SMLSTYPE {
	STYPE_Constant	= 0,	//!<   0 NUMBER
	STYPE_NumVar,			//!<   1 FVAR
	STYPE_StringVar,		//!<   2 SVAR
	STYPE_Array,			//!<   3 AVAR
	STYPE_Raster,			//!<   4 RVAR
	STYPE_Vector,			//!<   5 VVAR
	STYPE_CAD,				//!<   6 CVAR
	STYPE_Region,			//!<   7  region
	STYPE_TIN,				//!<   8 TINVAR
	STYPE_RESERVERD,		//!<   9 reserved for whatever
	STYPE_Keyword,			//!<  10
	STYPE_Intrinsic,		//!<  11 BLTIN
	STYPE_UFunction,		//!<  12 FUNCTION - User defined
	STYPE_UProcedure,		//!<  13 PROCEDURE - User defined
	STYPE_BFunction,		//!<  14 BFUNCTION - Built-in
	STYPE_BSFunction,		//!<  15 BSTRFUNC - Built-in
	STYPE_BProcedure,		//!<  16 PBROCEDURE - Built-in
	STYPE_Table,			//!<  17 TABLE
	STYPE_TableVar,		//!<  18 TVAR - pointer to table
	STYPE_AnyVal,			//!<  19 ANYVAL -- Either NUMBER or STRING
	STYPE_String,			//!<  20 String
	STYPE_AnyObject,		//!<  21	RVAR, VVAR, or CVAR
	STYPE_NumList,			//!<  22 NUMBER, VLVAR, or NUM_OR_LIST
	STYPE_StrList,			//!<  23 STRING, SLVAR, or STR_OR_LIST
	STYPE_List,				//!<  24 StrList or NumList
	STYPE_RecList,			//!<  25 May use NUMLIST instead
	STYPE_VectFunc,		//!<  26 Function returning an object
	STYPE_RESERVED27,		//!<  27 unused (used to be func returning region)
	STYPE_CADFunc,			//!<  28 Function returning an object
	STYPE_TINFunc,			//!<  29 Function returning an object
	STYPE_ClassRefFunc,	//!<  30 Function returning a class ptr that the caller should not free
	STYPE_ClassVar,		//!<  31 Variable holding a class ptr
	STYPE_ClassRef,		//!<  32 Just data that SML should NOT free
	STYPE_RastListVar,	//!<  33
	STYPE_VectListVar,	//!<  34
	STYPE_CADListVar,		//!<  35
	STYPE_TINListVar,		//!<  36
	STYPE_RegListVar,		//!<  37
	STYPE_Enum,				//!<  38
	STYPE_StrField,		//!<  39
	STYPE_NumField,		//!<  40
	STYPE_ClassHashFunc, //!<  41 Function returning an (SMLCLASSHASH*)
	STYPE_NumHashFunc,	//!<  42
	STYPE_FuncProto,		//!<  43
	STYPE_StrFuncProto,	//!<  44
	STYPE_ClassFuncProto,//!<  45
	STYPE_ProcProto,		//!<  46
	STYPE_ClassCopy,		//!<  47 Same as STYPE_ClassRef but needs to be freed
	STYPE_ClassCopyFunc,	//!<  48 Same as STYPE_ClassRefFunc need to free result
	STYPE_ClassHashVar,	//!<  49
	STYPE_NumHashVar,		//!<  50
	STYPE_Shape,			//!<  51
	STYPE_Style,			//!<  52
	STYPE_ClassConstRef,	//!<  53 For SMLCLASSMEMBERs, Same as STYLE_ClassRef, but "get" returns a const reference (NOTE! Not implemented yet, do not use)
	STYPE_ClassArray,		//!<  54 Pointer to SMLCLASSARRAY
	STYPE_ClassArrayVar,	//!<  55 Variable holding an STYPE_ClassArray
	STYPE_ClassArrayFunc,//!<  56 Function returning an SMLCLASSARRAY (NOT IMPLEMENTED)
	STYPE_MAXTYPE			//!<  57	 Always the last one
	};

namespace SML {
	typedef enum ::SMLSTYPE STYPE;
	}

#ifdef WIN32
#pragma deprecated(STYPE_Class)		// Use STYPE_ClassRef or STYPE_ClassCopy
#pragma deprecated(STYPE_ClassFunc) // Use STYPE_ClassRefFunc or STYPE_ClassCopyFunc
#endif
#define  STYPE_Class			STYPE_ClassRef
#define  STYPE_ClassFunc		STYPE_ClassRefFunc
#define	STYPE_StrVar			STYPE_StringVar
#define 	STYPE_Num				STYPE_Constant	//!<  Same as Constant



/*
** Flags returned by DBQUERY_STYPEFlags(stype).  They indicate what part
** of the SMLARG union is used by this STYPE
*/
#define STYPEFLAG_Val		0x0001
#define STYPEFLAG_Sym		0x0002
#define STYPEFLAG_Str		0x0004
#define STYPEFLAG_List		0x0008
#define STYPEFLAG_ValSym	0x0010
#define STYPEFLAG_StrSym	0x0020
#define STYPEFLAG_ObjSym	0x0040
#define STYPEFLAG_ListSym	0x0080
#define STYPEFLAG_FuncSym	0x0100
#define STYPEFLAG_ArraySym	0x0200
#define STYPEFLAG_ClassSym	0x0400
#define STYPEFLAG_Prototype 0x0800
#define STYPEFLAG_ClassPtr	0x1000

#endif		//  INC_SML_STYPE_H

