/**
 * \file spatmod/imagesamplegroup.h
 * \brief SPATMOD::IMAGE::SAMPLEGROUP definitions
 *
 * \if NODOC
 * $Id: imagesamplegroup.h_v 1.2 2012/12/11 18:53:15 mju Exp $
 *
 * $Log: imagesamplegroup.h_v $
 * Revision 1.2  2012/12/11 18:53:15  mju
 * Add getIdxOpacity.
 *
 * Revision 1.1  2012/12/11 17:46:05  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESAMPLEGROUP_H
#define	INC_SPATMOD_IMAGESAMPLEGROUP_H

#ifndef	INC_SPATMOD_DATATYPE_H
#include <spatmod/datatype.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//! Container for "group" (range) of sample indices with associated opacity and data type.
//! Depending on how the group is constructed it may or may not include the opacity index in its range.
class SAMPLEGROUP {
	public:

		//! Default constructor.
		SAMPLEGROUP (
			): m_IdxFirst(UINT16_MAX), m_IdxLast(0), m_IdxOpacity(UINT16_MAX), m_DataType(DATATYPE_Invalid) { }

		//! Construct with range, opacity index and datatype.
		SAMPLEGROUP (
			UINT16 IdxFirst,
			UINT16 IdxLast,
			UINT16 IdxOpacity,
			DATATYPE DataType
			): m_IdxFirst(IdxFirst), m_IdxLast(IdxLast), m_IdxOpacity(IdxOpacity), m_DataType(DataType) { }

		//! Construct with range and datatype (no opacity).
		SAMPLEGROUP (
			UINT16 IdxFirst,
			UINT16 IdxLast,
			DATATYPE DataType
			): m_IdxFirst(IdxFirst), m_IdxLast(IdxLast), m_IdxOpacity(UINT16_MAX), m_DataType(DataType) { }

		//! Get sample data type if all same.
		DATATYPE GetDataType () const
			{ return (m_DataType); }

		//! Get index of first sample in group.
		UINT16 GetIdxFirst () const
			{ return (m_IdxFirst); }

		//! Get index of last sample in group.
		UINT16 GetIdxLast () const
			{ return (m_IdxLast); }

		//! Get index of opacity sample for group.
		UINT16 GetIdxOpacity () const
			{ return (m_IdxOpacity); }

		//! Determine if has valid opacity sample index.
		bool HasOpacity () const
			{ return (m_IdxOpacity != UINT16_MAX); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT16 m_IdxFirst;
		UINT16 m_IdxLast;
		UINT16 m_IdxOpacity;
		DATATYPE m_DataType;
		#endif
	};

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESAMPLEGROUP_H
